############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
Provides a simple interface to the myproxy module.
"""

from pyGlobus.util import GlobusException 
import myProxyc
import os
import utilc, gsic

DEFAULT_CERTFILE ="~/.globus/usercert.pem"
DEFAULT_OUTFILE = "/tmp/x509up_u"


class MyProxyException(GlobusException):
    """
       A simple subclass of GlobusException
       
       Used for errors in the myProxy module. It defines no new methods.
       
       **Extends:**
       
            *pyGlobus.util.GlobusException*
       """
    pass

class MyProxy:

    def __init__(self):
	"""
	Initializes the memory for the underlying structures

	**Arguments**

	None

	**Returns**

	None

	**Raises**
	A MyProxyException is raised if there is a problem initializing
	the underlying structures
	"""
	
	attrs, request, response = myProxyc.myproxy_init()
	
	ret, attrs, request = myProxyc.set_delegation(attrs, request)
	if not ret == 0:
	    ex = MyProxyException(attrs)
	    raise ex

	self._attrs = attrs
	self._request = request
	self._response = response

    def __del__(self):
	"""
	Frees the memory associated with underlying structures
	"""
	
	myProxyc.myproxy_destroy(self._attrs, self._request, self._response)

    def create_proxy(self,proxyFile, hostname, port = 7512, lifetime=12*3600):
	"""
	Create a proxy and send it to the myproxy server

	**Arguments**

	- *hostname* a string of the hostname of the myproxy server

	- *port* an int of the port the myproxy server is listening on

	- *username* the username of who is creating the proxy

	**Returns**

	None

	***XXX*** This method deactivates all globus modules, this is necessary because
	the myproxy libraries activate globus modules then later do not deactivate them.
	
	**Raises**
	A MyProxyException is raised if there is a problem creating the proxy
	"""
	
	
	
	username = os.getlogin()
	ret = myProxyc.proxy_init(self._attrs, self._request,
				  self._response,hostname, port,
				  lifetime,proxyFile,
				  username)
        if not ret == 0:
            ex = MyProxyException("Problem initializing proxy")
            raise ex

    def get_info(self,hostname, port =7512,dnAsUsername=None):
	"""
	Gets information about proxy from the myproxy repository

	**Arguments**

	- *hostname* a string which is the hostname of the repository
	
	- *port* an int which is the port the repository is listening on

	- *dnAsUsername* The DN of the proxy whom you want to query the server
	about
	
	**Returns**
	A string of the proxy information

	**Raises**
	A MyProxyException is raised if there is a problem getting the credential
	"""
	utilc.globus_module_activate(gsic.get_module())
	if not dnAsUsername:
	    useDnAsUsername = 0
	    dnAsUsername = os.getlogin()
	else:
	    useDnAsUsername = 1
	    
	username = dnAsUsername    
	ret, proxyString = myProxyc.proxy_info(self._attrs, self._request,
				  self._response,hostname,port,
				  username, useDnAsUsername)

	if not ret == 0:
	    ex = MyProxyException(proxyString)
	    utilc.globus_module_deactivate(gsic.get_module())
	    raise ex

	
	return proxyString
    
    def destroy_proxy(self,hostname, port =7512,dnAsUsername=None):
	"""
	Destroys about proxy from the myproxy repository

	**Arguments**

	- *hostname* a string which is the hostname of the repository
	
	- *port* an int which is the port the repository is listening on

	- *dnAsUsername* The DN of the proxy whom you want to query the server
	about
	
	**Returns**
	None

	**Raises**
	A MyProxyException is raised if there is a problem getting the credential
	"""
	utilc.globus_module_activate(gsic.get_module())
	if not dnAsUsername:
	    useDnAsUsername = 0
	    dnAsUsername = os.getlogin()
	else:
	    useDnAsUsername = 1
	    
	username = dnAsUsername    
	ret, errorString = myProxyc.proxy_destroy(self._attrs, self._request,
				  self._response,hostname,port,
				  username, useDnAsUsername)
	if not ret == 0:
	    ex = MyProxyException(errorString)
	    utilc.globus_module_deactivate(gsic.get_module())
	    raise ex
	utilc.globus_module_activate(gsic.get_module())
	
    def get_delegation(self, hostname, port = 7512, certfile = None,
		       outfile = None, lifetime = 12 *3600):
	"""
	Gets the credential from the myproxy repository

	**Arguments**

	- *hostname* a string which is the hostname of the repository

	- *outfile* a string which is the file where the credential will be written to
	
	- *port* an int which is the port the repository is listening on

	- *certfile* the location of the certfile

	- *lifetime* an int which is the number of seconds the credential is valid

	**Returns**
	None

	**Raises**
	A MyProxyException is raised if there is a problem getting the credential
	"""

	username = os.getlogin()
	if certfile == None:
	    certfile = DEFAULT_CERTFILE
	if outfile == None:
	    outfile = DEFAULT_OUTFILE+ str(os.getuid())

	utilc.globus_module_activate(gsic.get_module())
	ret = myProxyc.get_delegation(self._attrs, self._request,
				      self._response,hostname,port,
				      certfile, outfile,
				      lifetime, username)

	utilc.globus_module_deactivate(gsic.get_module())
	if not ret == 0:
	    ex = MyProxyException("Failed to receive proxy")
	    raise ex
    
	



	

    
