############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
The main interface to the Globus Replica Management functionality.
"""

from pyGlobus import utilc
from pyGlobus.util import GlobusException
from pyGlobus.ftpClient import HandleAttr
import replicaManagementc

#############################################################
# Module Functions
############################################################


##########################################################
# Module Classes
##########################################################

class SessionException(GlobusException):
    """
    A simple subclass of GlobusException.

    Used for errors in the replica management session module. It defines no new methods.

    **Extends:**
    
        *pyGlobus.util.GlobusException*
    """
    pass

class SessionAttrException(GlobusException):
    """
    A simple subclass of GlobusException.

    Used for errors in the replica management session attribute module. It defines no new methods.

    **Extends:**
    
        *pyGlobus.util.GlobusException*
    """
    pass

class Session:
    """
    The main interface to the replica catatlog session module.

    It is mainly a proxy class for the replicaManagementc module session functions. It provides a higher
    level interface designed to simplify usage.  Exceptions are used to indicate
    all error conditions.
    """
    
    def __init__(self, attr=None):
        """
        Initialize a session.
        
        This will activate the globus_replica_management_module, and initialize the session
        handle.

        **Arguments**

            - *attr* SessionAttr object containg the attributes for the session
            handle. If None, default attributes will be used.
        
        **Raises:**
        
            A SessionException is thrown if unable to activate the module, or
            init the handle.
        """
        self._handle = None
        self._cbList = []
        ret = utilc.globus_module_activate(replicaManagementc.get_module())
        if not ret == 0:
            ex = SessionException("Unable to initialize the module")
            raise ex
        if attr == None:
            ret, attrPtr = replicaManagementc.handleattr_init()
            if not ret == 0:
                ex = SessionException(attrPtr)
                raise ex
            attr = SessionAttr(attrPtr)
        self._attr = attr
        ret, handle = replicaManagementc.handle_init(attr._handle)
        if not ret == 0:
            ex = SessionException(handle)
            raise ex
        self._handle = handle
        return

    def __del__(self):
        """
        Destroys an instance.

        It will destroy the handle, and deactivate the globus_replica_management
        module when the object is destroyed.

        **Raises:**
        
            A SessionException is thrown if unable to destroy the module. This
            exception can't be caught, and should never happend.  It's there mainly
            for debugging if it does happen. :-)
        """
        for cb in self._cbList:
            self.free_callback(cb)
        if not self._handle == None:
            ret, err = replicaManagementc.handle_destroy(self._handle)
            if not ret == 0:
                ex = SessionException(err)
                utilc.globus_module_deactivate(replicaManagementc.get_module())
                raise ex
        ret = utilc.globus_module_deactivate(replicaManagementc.get_module())
        if not ret == 0:
            ex = SessionException("Unable to deactivate the module")
            raise ex
        return

    def restart(self, rollbackInfo, colAttr=None, srcFtpAt=None,
                destFtpAttr=None, preregFunc=None, cbArg=None):
        """
        Restart the operation described by the given lock.

        **Arguments**

            - *rollbackInfo* an opaque contact string containing a pointer to
            rollback and restart information for the operation to be restarted.

            - *colAttr* A CollectionAttr containing the replica catalog
            collection attribute, if None, default attributes are used.

            - *srcFtpAt* A FtpClientOperationAttr object containg the
            attributes for the source url, if None, default attributes are used.

            - *destFtpAttr* A FtpClientOperationAttr object containing teh
            attributes for the destination url, if None, default attributes are used.

            - *preregFunc*  a user-supplied function called between copying
            the file and registering the file in the replica catalog, the function
            must not alter the file or the replica catalog and must be able to be
            called multiple times as there may be multiple restart attempts. The
            callback has the following form:

            'function(arg, url)' where arg is hte arg passed into this funcion,
            and url is the destination url of the just completed copy. Returns
            0, or a globus_result_t error object.

            This function must not

            * modify the file that has been published or copied
            * modify information in the replica catalog

            This function must
            
            * be thread safe
            * be able to be called multiple times (for example, during multiple restart attempts)

            No attempt is made to roll back any effects of this function.

            - *cbArg* A argument to be passed to the preregFunc. 

        **Raises**

            A SessionException is thrown if unable to restart the operation.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        saPtr = None
        if not srcFtpAttr == None:
            saPtr = srcFtpAttr._handle
        daPtr = None
        if not destFtpAttr == None:
            daPtr = destFtpAttr._handle
        ret, cbHandle = replicaManagementc.restart(self._handle, rollbackInfo,
                                                   caPtr, saPtr, daPtr, preregFunc, cbArg)
        if not ret== 0:
            ex = SessionException(cbHandle)
            raise ex
        if not preregFunc == None:
            self._cbList.append(cbHandle)
        return

    def rollback(self, rollbackInfo, colAttr=None, ftpAttr=None):
        """
        Rollback the operation described by the given lock.

        **Arguments**

            - *rollbackInfo* an opaque string containing a
            contact string for rollback and restart information
            for an operation to be rolled back

            - *colAttr* A CollectionAttr containing the replica catalog
            collection attribute, if None, default attributes are used.

            - *ftpAttr* A FtpClientOperationAttr object containg the
            attributes for the destination url, if None, default attributes are used.

        **Raises**

            A SessionException is thrown if unable rollback the
            operation.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        faPtr = None
        if not ftpAttr ==None:
            faPtr = ftpAttr._handle
        ret, err = replicaManagementc.rollback(self._handle,
                                               rollbackInfo, caPtr, faPtr)
        if not ret == 0:
            ex = SessionException(err)
            raise ex
        return

    def get_rollback_contact(self):
        """
        Get rollback string.
        
        Obtain the rollback and restart information contact string from the
        result of a replica management operation.

        he operations that support rollback or restart are
        file_publish(), file_copy(), and file_delete().

        **Returns**

            The rollback and restart information contact string
            associated with the operation.

        **Raises**

            A SessionException is thrown if unable to get the rollback
            contact.
        """
        ret, contact = replicaManagementc.get_rollback_contact(self._handle)
        if not ret == 0:
            ex = SessionException(contact)
            raise ex
        return contact

    def collection_create(self, url, colAttr):
        """
         Create an empty collection.

         The collection is created with no filenames and default access
         control rights as defined by the administrator of the replica catalog.

         The caller must be authorized to create the collection entry in the
         replica catalog.

         **Arguments**

             - *url* The string LDAP URL of the collection to add.

             - *colAttr* A CollectionAttr object containg the replica catalog
             collection attributes.

        **Raises**

            A SessionException is thrown if unable to create the
            collection.
        """
        ret, err = replicaManagementc.collection_create(self._handle,
                                                        url, colAttr._handle)
        if not ret == 0:
            ex = SessionException(err)
            raise ex
        return

    def collection_delete(self, url, colAttr):
        """
        Delete a collection with no filenames.

        The caller must be authorized to delete the collection
        entry in the replica catalog.

        **Arguments**

            - *url* The string LDAP URL of the collection to delete.

             - *colAttr* A CollectionAttr object containg the replica catalog
             collection attributes.

        **Raises**

            A SessionException is thrown if unable to delete the
            collection.
        """
        ret, err = replicaManagementc.collection_delete(self._handle,
                                                        url, colAttr._handle)
        if not ret == 0:
            ex = SessionException(err)
            raise ex
        return

    def location_create(self, url, name, rootURL, colAttr=None,
                        ftpAttr=None):
        """
        Create a storage system location in the replica catalog,

        The location is created without any associated filenames.

        The caller must be authorized to create the storage system directory
        and to add a location entry to the collection in the replica catalog.

        **Arguments**

            - *url* The string URL of the collection to add the location
            to.

            - *name* A string unique name of a location to add within the
            replica catalog collection.

            - * rootURL* A string URL of the root directory in the directory
            system where the files of this collection will reside, for
            example, "gsiftp://host:port/dir".

            - *colAttr* A CollectionAttr object containg the attributes
            for the collection. If None, default attributes are used.

            - *ftpAttr* A FtpClientOperationAttr containg the attributes
            associated with the location. If None, default attributes
            are used.

        **Raises**

            A SessionException is thrown if unable to create the
            location.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        faPtr = None
        if not faPtr == None:
            faPtr = ftpAttr._handle
        ret, err = replicaManagementc.location_create(self._handle,
                                                      url, caPtr, name, faPtr, rootURL)
        if not ret == 0:
            ex = SessionException(err)
            raise ex
        return

    def location_delete(self, url, name, colAttr=None):
        """
        Delete a storage system location from the replica catalog.

        The caller must be authorized to remove the location entry from the
        collection in the replica catalog. Note that this call will fail if there
        are any files associated with the location and that the associated
        storage system will not be affected.

        **Arguments**

            - *url* A string ldap url of the collection to remove the
            location from.

            - *name* The name of a location to remove from the
            replica catalog collection.

            - *colAttr* A CollectionAttr object containg the attributes
            for the collection. If None, default attributes are used.

        **Raises**

            A SessionException is thrown if unable to delete the location.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        ret, err = replicaManagementc.location_delete(self._handle,
                                                      url, caPtr, name)
        if not ret == 0:
            ex = SessionException(err)
            raise ex
        return

    def file_register(self, url, locationName, filename,
                      colAttr=None, ftpAttr=None):
        """
        Register a file into a collection and location.
        
        Registers an existing file in a storage system into an existing
        replica catalog collection and a location.

        The file already exists in a storage system referred to by a replica
        catalog location, no copying is done.
        
        The caller must have authorization to add a filename to the replica
        catalog collection and location, and to read the file from the storage system.

        **Arguments**

            - *url* A string LDAP URL of an existing replica catalog
            collection to add the file to.

            - *locationName* A string name of an existing replica catalog
            location within the replica catalog collection to add the file to.

            - *filename* A string logical file name of the file to add must exist in
            the storage system directory referred to by the replica catalog
            location's URL constructor

            - *colAttr* A CollectionAttr object containg the attributes
            for the collection. If None, default attributes are used.

            - *ftpAttr* A FtpClientOperationAttr containg the attributes
            associated with the location. If None, default attributes
            are used.

        **Raises**

            A SessionException is thrown if unable to register the
            file.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        faPtr = None
        if not ftpAttr == None:
            faPtr = ftpAttr._handle
        ret, err = replicaManagmentc.file_register(self._handle,
                                                   url, caPtr, locationName, faPtr, filename)
        if not ret == 0:
            ex = SessionException(err)
            raise ex
        return

    def file_publish(self, srcUrl, destUrl, locationName,
                     filename, behavior, srcAttr=None, colAttr=None,
                     destAttr=None, preregFunc=None, cbArg=None):
        """
         Publish a file into a replica catalog collection and location.

         Copies the file from its source into the replica catalog location's storage system.
         
         Throughout, the replica catalog remains consistent: any files listed in a replica catalog
         location are complete and uncorrupt (barring spontaneous corruption by a storage system).
         Pending operations on files are locked and their state is checkpointed to the replica catalog
         so that, in the event of failure, rollback and restart information is available.

         The caller must have authorization to add a filename to the replica catalog collection and
         location, to write the file to the storage system, and to read the source file using their own
         credentials.

        **Arguments**

            - *srcUrl* A string URL of the file to be published into the replica catalog and copied
            to the replica catalog location's storage system, the URL can have the protocol
            schemes file: , ftp: , gsiftp: .

            - *destUrl* A string LDAP URL of an existing replica catalog collection to add the file to.

            - *locationName* A string name of an existing replica catalog location in the replica
            catalog collection to add the file to

            - *filename* A string containg logical file name of the file to add.

            - *behavior* The overwrite behavior if the file already exists on the destination
            storage system, if an error occurs when specifying:
            * GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_DISALLOW
            then neither the file nor the replica catalog are modified;
            * GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_SAFE
            then the original file is left intact (and is available via the replica catalog throughout the transfer); or
            * GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_UNSAFE
            then the original file is corrupted and a rollback would attempt to remove the original file
            (which is unavailable via the replica catalog throughout the copying operation)

            - *srcAttr* A FtpClientOperationAttr object containg the
            attributes for the source url, if None, default attributes are used.
            
            - *colAttr* A CollectionAttr containing the replica catalog
            collection attribute, if None, default attributes are used.

            - *destAttr* A FtpClientOperationAttr object containing the
            attributes for the destination url, if None, default attributes are used.

            - *preregFunc*  a user-supplied function called between copying
            the file and registering the file in the replica catalog, the function
            must not alter the file or the replica catalog and must be able to be
            called multiple times as there may be multiple restart attempts. The
            callback has the following form:

            'function(arg, url)' where arg is hte arg passed into this funcion,
            and url is the destination url of the just completed copy. Returns
            0, or a globus_result_t error object.

            This function must not

            * modify the file that has been published or copied
            * modify information in the replica catalog

            This function must
            
            * be thread safe
            * be able to be called multiple times (for example, during multiple restart attempts)

            No attempt is made to roll back any effects of this function.

            - *cbArg* A argument to be passed to the preregFunc.

        **Raises**

            A SessionException is thrown if unable to publish the file.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        saPtr = None
        if not srcAttr == None:
            saPtr = srcAttr._handle
        daPtr = None
        if not destAttr == None:
            daPtr = destAttr._handle
        ret, cbHandle = replicaManagementc.file_publish(self._handle, srcUrl,
                                                        saPtr, destUrl, caPtr, locationName, daPtr,
                                                        filename, behavior, preregFunc, arg)
        if not ret== 0:
            ex = SessionException(cbHandle)
            raise ex
        if not preregFunc == None:
            self._cbList.append(cbHandle)
        return

    def file_copy(self, url, srcLocationName, destLocationName,
                  filename, behavior, colAttr=None, srcAttr=None,
                  destAttr=None, preregFunc=None, cbArg=None):
        """
         Copy a file from one replica catalog location's storage system to another.

         Updates the replica catalog collection and location information..

         Throughout, the replica catalog remains consistent: any files listed in a replica catalog
         location are complete and uncorrupt (barring spontaneous corruption by a storage system).
         Pending operations on files are locked and their state is checkpointed to the replica
         catalog so that, in the event of failure, rollback and restart information is available.

         The caller must have authorization to add a filename to the replica catalog collection and
         location, to write the file to the destination storage system, and to read from the source
         storage system.

         **Arguments**

             - *url* A string LDAP URL of an existing replica catalog collection to copy the file in.

             - *srcLocationName* A string name of an existing replica catalog location in the replica
             catalog collection to copy the file from.

             - *destLocationName* A string name of an existing replica catalog location in the replica
             catalog collection to add the file to.

             - *filename* The string filename of the file to add, which must exist in the storage system
             directory referred to by the replica catalog locations's URL constructor and must match
             the file specification for this replica catalog location

             - *behavior* The overwrite behavior if the file already exists on the destination storage
             system, if an error occurs when specifying:
             * GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_DISALLOW
             then neither the file nor the replica catalog are modified;
             * GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_SAFE
             then the original file is left intact (and is available via the replica catalog
             throughout the transfer); or
             * GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_UNSAFE
             then the original file is corrupted and a rollback would attempt to remove the original file
             (which is unavailable via the replica catalog throughout the copying operation)

             - *colAttr* A CollectionAttr containing the replica catalog
            collection attribute, if None, default attributes are used.

            - *srcAttr* A FtpClientOperationAttr object containg the
            attributes for the source url, if None, default attributes are used.
            
            - *destAttr* A FtpClientOperationAttr object containing the
            attributes for the destination url, if None, default attributes are used.

            - *preregFunc*  a user-supplied function called between copying
            the file and registering the file in the replica catalog, the function
            must not alter the file or the replica catalog and must be able to be
            called multiple times as there may be multiple restart attempts. The
            callback has the following form:

            'function(arg, url)' where arg is hte arg passed into this funcion,
            and url is the destination url of the just completed copy. Returns
            0, or a globus_result_t error object.

            This function must not

            * modify the file that has been published or copied
            * modify information in the replica catalog

            This function must
            
            * be thread safe
            * be able to be called multiple times (for example, during multiple restart attempts)

            No attempt is made to roll back any effects of this function.

            - *cbArg* A argument to be passed to the preregFunc.

        **Raises**

            A SessionException is thrown if unable to copy the file.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        saPtr = None
        if not srcAttr == None:
            saPtr = srcAttr._handle
        daPtr = None
        if not destAttr == None:
            daPtr = destAttr._handle
        ret, cbHandle = replicaManagementc.file_copy(self._handle, url, caPtr,
                                                     srcLocationName, saPtr, destLocationName,
                                                     daPtr, filename, behavior, preregFunc, cbArg)
        if not ret == 0:
            ex = SessionException(cbHandle)
            raise ex
        if not preregFunc == None:
            self._cbList.append(cbHandle)
        return

    def file_delete(self, url, locationName, filename, delete,
                    colAttr=None, ftpAttr=None):
        """
        Remove a filename from the replica catalog location.

        Also removes the file from the replica catalog location's storage system.

        If delete is not true then don't also delete the file from the replica catalog
        location's storage system.

        The caller must have authorization to remove a filename from the replica catalog
        location and to delete the file from the source storage system.

        **Arguments**

            - *url* A string LDAP URL of an existing replica catalog collection to
            remove the file from.

            - *locationName* A string name of an existing replica catalog location in the
            replica catalog collection to remove the file from.

            - *filename* A string filename of the file to remove, which must exist in the
            storage system directory referred to by the replica catalog locations's
            URL constructor.

            - *delete* Whether to delete the file from the replica catalog
            location's storage system.

            - *colAttr* A CollectionAttr containing the replica catalog
            collection attribute, if None, default attributes are used.

            - *ftpAttr* A FtpClientOperationAttr object containg the
            attributes for the source url, if None, default attributes are used.

        **Raises**

            A SessionException is thrown if unable to delete the file.
        """
        caPtr = None
        if not colAttr == None:
            caPtr = colAttr._handle
        saPtr = None
        if not ftpAttr == None:
            saPtr = srcAttr._handle
        ret, err = replicaManagementc.file_delete(self._handle, url, caPtr,
                                                  locationName, saPtr, filename, delete)
        if not ret == 0:
            ex = Session(err)
            raise ex
        return

    def free_callback(self, cbHandle):
        """
        Free a callback.

        Free's the underlying memory allocated for a callback structure.

        **Arguments:**

            - *cbHandle* A SWIG'ized pointer to a callback structure.
        """
        replicaManagmentc.free_callback(cbHandle)
        indx = self._cbList.index(cbHandle)
        del self._cbList[indx]
        return
    
    def get_handle(self):
        """
        Return the underlying replica management handle.

        **XXX** This handle is only valid as long as this object exists.  When
        the object is destroyed, the memory this handle points to is freed.

        **Returns:**

            A string containing the pointer.
        """
        return self._handle

class SessionAttr:
    """
    The main interface to the replica management session attribute module.

    t provides get and set methods for all of the various handle
    specific attributes, and hides the underlying memory management.
    """
    
    def __init__(self, handle=None):
        """
        Construct an instance.
        
        This will activate the globus_replica_management_module, and create
        the session attr handle..

        **Arguments**

            - *handle* A SWIG'ized pointer to a globus_replica_management_handleattr.
        
        **Raises:**
        
            A SessionAttrException is thrown if unable to activate the module, or
            create the attribute.
        """
        self._activated = 0
        self._handle = None
        ret = utilc.globus_module_activate(replicaManagementc.get_module())
        if not ret == 0:
            ex = SessionAttrException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            ret, handle = replicaManagementc.handleattr_init()
            if not ret == 0:
                ex = SessionAttrException(handle)
                raise ex
        self._handle = handle
        return

    def __del__(self):
        """
        Destroys an instance.

        It will close the connection, and deactivate the globus_replica_management
        module, and delete the handle.

        **Raises:**
        
            A SessionAttrException is thrown if unable to destroy the module. This
            exception can't be caught, and should never happend.  It's there mainly
            for debugging if it does happen. :-)
        """
        if not self._handle == None:
            ret, err = replicaManagementc.handleattr_destroy(self._handle)
            if not ret == 0:
                ex = SessionAttrException(err)
                raise ex
        ret = utilc.globus_module_deactivate(replicaManagementc.get_module())
        if not ret == 0:
            ex = SessionAttrException("Unable to deactivate the module")
            raise ex
        return

    def copy(self):
        """
        Copy the session attributes.

        **Returns**

            A new SessionAttr instance with the same setting as self.

        **Raisess**

            A SessionAttrException is thrown if unable to copy the attr.
        """
        ret, new = replicaManagementc.handleattr_copy(self._handle)
        if not ret == 0:
            ex = SessionAttrException(new)
            raise ex
        return SessionAttr(new)

    def set_caching(self, caching):
        """
        Set the catalog connection caching behavior.

        **Arguments**

            - *caching* boolean, whether to enable caching.

        **Raises**

            A SessionAttrException is thrown if unable to set the
            caching behavior.
        """
        ret, err = replicaManagementc.handleattr_set_caching(self._handle, caching)
        if not ret == 0:
            ex = SessionAttrException(err)
            raise ex
        return

    def get_caching(self):
        """
        Get the catalog connection caching behavior.

        **Returns**

            A boolean, true if caching is enable, false otherwise.

        **Raises**

            A SessionAttrException is thrown if unable to get the caching
            behavior.
        """
        ret = replicaManagementc.handleattr_get_caching(self._handle)
        if not ret[0] == 0:
            ex = SessionAttrException(ret[1])
            raise ex
        return ret[2]

    def set_ftp_update_frequency(self, frequency):
        """
        Set the rollback and restart information update frequency.

        **Arguments**

            - *frequency*  how often the rollback and restart information
            is generated during a transfer is pushed out to the rollback
            and restart information entry, expressed as a multiple of 5
            second intervals

        **Raises**

            A SessionAttrException is thrown if unable to set the update
            frequency.
        """
        ret, err = replicaManagementc.handleattr_set_ftp_update_frequency(self._handle,
                                                                          frequency)
        if not ret == 0:
            ex = SessionAttrException(err)
            raise ex
        return

    def get_ftp_update_frequency(self):
        """
        Get the rollback and restart information update frequency.

        **Returns**

            how often the rollback and restart information
            is generated during a transfer is pushed out to the rollback
            and restart information entry, expressed as a multiple of 5
            second intervals

        **Raises**

            A SessionAttrException is thrown if unable to get the update
            frequency.
        """
        ret = replicaManagementc.handleattr_get_ftp_update_frequency(self._handle)
        if not ret[0] == 0:
            ex = SessionAttrException(ret[1])
            raise ex
        return ret[1]

    def set_ftp_handleattr(self, ftpAttr):
        """
        Set the GridFTP handle attributes.

        These attributes are used when initializing the GridFTP handle.

        **Arguments**

            - *ftpAttr* A HandleAttr instance.

        **Raises**

            A SessionAttrException is thrown if unable to set the ftp
            handleattr.
        """
        ret, err = replicaManagementc.handleattr_set_ftp_handleattr(self._handle,
                                                                    ftpAttr._handle)
        if not ret == 0:
            ex = SessionAttrException(err)
            raise ex
        return

    def get_ftp_handleattr(self):
        """
        Get the GridFTP handle attributes.

        These attributes are used when initializing the GridFTP handle.

        **Returns**

            A HandleAttr instance.

        **Raises**

            A SessionAttrException is thrown if unable to get the ftp
            handleattr.
        """
        ret, handle = replicaManagementc.handleattr_get_ftp_handleattr(self._handle)
        if not ret == 0:
            ex = SessionAttrException(handle)
            raise ex
        return HandleAttr(handle, 0)
    
    def set_third_party_transfer(self, third_party_transfer):
        """
        Set whether to initiate an FTP third party transfer or to buffer the transfer locally.

        he default is to use third party transfer.

        **Arguments**

            - *third_party_transfer* boolean, whether to enable third_party_transfer.

        **Raises**

            A SessionAttrException is thrown if unable to set the
            third_party_transfer behavior.
        """
        ret, err = replicaManagementc.handleattr_set_third_party_transfer(self._handle,
                                                                          third_party_transfer)
        if not ret == 0:
            ex = SessionAttrException(err)
            raise ex
        return

    def get_third_party_transfer(self):
        """
        Get the catalog connection third_party_transfer behavior.

        **Returns**

            A boolean, true if third_party_transfer is enable, false otherwise.

        **Raises**

            A SessionAttrException is thrown if unable to get the third_party_transfer
            behavior.
        """
        ret = replicaManagementc.handleattr_get_third_party_transfer(self._handle)
        if not ret[0] == 0:
            ex = SessionAttrException(ret[1])
            raise ex
        return ret[2]
        
    def get_handle(self):
        """
        Return the underlying replica management handle attr handle.

        **XXX** This handle is only valid as long as this object exists.  When
        the object is destroyed, the memory this handle points to is freed.

        **Returns:**

            A string containing the pointer.
        """
        return self._handle
