############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
A module to provide some of the shared GSI functionality.
"""
from pyGlobus import gsic
from pyGlobus import utilc
from pyGlobus.util import GlobusException

class SecuritySingleton:
    """
       Activates the security module once no matter how many
       instances are created.

       This class is a hack due to problems deactivating GSS_ASSIST_MODULE
       """
    activated = 0
    instances = 0
    def __init__(self):
        if not SecuritySingleton.activated:
            
            ret = utilc.globus_module_activate(gsic.get_module())
            if not ret == 0:
                raise GlobusException("Unable to activate the security module")
            
            SecuritySingleton.activated = 1
        SecuritySingleton.instances +=1

    def __del__(self):
        SecuritySingleton.instances -=1

        
#########################################################################
# Module functions
########################################################################

def gss_no_cred_factory():
    """
    Returns a GSSCred containing GSS_C_NO_CRED.

    GSS_C_NO_CRED is used by many GSI calls to indicate the usage of the default credential.
    """
    secureModule = SecuritySingleton()
    cred = GSSCred(gsic.get_no_cred_const_ptr())
    return cred  
    


def grid_proxy_init(verbose=1):
    """
    Creates a user proxy in the /tmp/x509uuid

    Writes the credential to /tmp/x509uid

    *Raises*
    a GSIException is raised if the user proxy cannot be created
    
    *Returns*
    None
    """
    secureModule = SecuritySingleton()
    ret, cred = gsic.grid_proxy_init(verbose)
    if ret != 0:
        raise GSIException(cred)

def grid_proxy_info(verbose=1):
    """
    Retrieves information about the user proxy 

    *Raises*
    a GSIException is raised if information about the user proxy cannot
    be retrieved
       
    *Returns*
    a string with all of the user proxy information
    """

    secureModule = SecuritySingleton() 
    ret, proxy_string = gsic.grid_proxy_info(verbose) 
    if not ret == 0:
        raise GSIException(proxy_string)

    return proxy_string


def grid_proxy_destroy():
    """
    Destroys the user proxy in the default location or
    in the X509_USER_PROXY
    
    *Raises*
    a GSIException is raised if the user proxy cannot be destroyed

    *Returns*
    None
    """

    secureModule = SecuritySingleton()
    ret, error_string = gsic.grid_proxy_destroy() 
    if not ret == 0:
        raise GSIException(error_string)

def gridmap(globusid):
    """ 
    Returns the userid if it is in the grid-mapfile
    
    If you are a non-root user it will look in ~/.gridmap first

    **Arguments**

    *globusid*
    a string of Subject: line in a usercert
    
    *Returns*
    the userid that globusid maps to

    *Raises*
     a GSIException is raised if the globusid is not in the gridmap file
    
    """
    secureModule = SecuritySingleton()
    ret, userid = gsic.assist_gridmap(globusid)
   
    if not ret == 0:
        raise GSIException, "gridmap call failed"

    return userid

def userok(globusid, userid):
    """
    Returns 1 if the globusid maps to the userid in the ~/.gridmap file
    """

    secureModule = SecuritySingleton()
    ret = gsic.assist_userok(globusid, userid)
    
    
    if ret == -1:
        raise GSIException, "userok call failed"
    return 1

def map_local_user(local_user):
    """
       Returns the DN of that the local account maps to.
       """
    secureModule = SecuritySingleton()
    ret, globususer = gsic.assist_map_local_user(local_user)

    if not ret == 0:
        raise GSIException, "map_local_user call failed"

    return globususer
    
    
def genreq(issuer_gsscred, sub_gsscred, X509_request, private_key, bits=512):
    """
    Creates a X509_REQ and EVP_PKEY object, gsscred issuer's name is used
    for generation of gsscred sub's name.  gsscred sub object used for ??
    
    Not supported for gt-2.2
    """
    secureModule = SecuritySingleton()
     
    ret = gsic.genreq(issuer_gsscred._handle,
                           sub_gsscred._handle,
                           X509_request._handle,
                           private_key._handle, bits)
     

    if ret < 0:
        raise GSIException, 'generate request failed'
    return    

def sign_request(icred, scred, req):
    """
       Not supported for gt-2.2
       """
    
    print "icred: ", icred._handle
    print "scred: ", scred._handle
    print "req:   ", req._handle
    secureModule = SecuritySingleton()
    cred = gsic.sign_request(icred._handle, scred._handle, req._handle)
 
    return cred
                             

###########################################################################
# Module Classes
##########################################################################

class GSIException(GlobusException):
    """
       A sub-class of GlobusException.

       Used as a base class for all other security exceptions.

       **Extends:**

       *pyGlobus.util.GlobusException*
       """
    
    pass

class GSSContextException(GSIException):
    """
       A sub-class of GSSContextException.

       Used to indicate errors in the GSSContext module.

       **Extends:**

       *pyGlobus.security.GSI.GSIException*
       """
    
    pass

class GSSNameException(GSIException):
    """
       A sub-class of GSSContextException.

       Used to indicate errors in the GSSName module.
       
       **Extends:**

        *pyGlobus.security.GSI.GSIException*
        """
    
    pass

class GSSCredException(GSIException):
    """
       A sub-class of GSIException.

       Used to indicate errors in the GSSCred module.

       **Extends:**

       *pyGlobus.security.GSI.GSIException*
       """
    
    pass


class GSSMechsException(GSIException):
    """
       A sub-class of GSIException.
       
       Used to indicate errors in the GSSMechs module.

       **Extends:**

       *pyGlobus.security.GSI.GSIException*
       """
    
    pass

class GSSContext:
    """
       A  class for the underlying gss_ctx_id_t object.
       """

    def __init__(self, handle=None, free=1):
        """
              Constructs an instance.
              
              Each instance wraps a gss_ctx_id_t object.
              Defaults to a GSS_C_NO_CONTEXT

              **Arguments:**
              
              - *handle* An optional argument that, if present, should
              contain a SWIG'ized pointer to a gss_ctx_id_t object.

              - *free* Should the underlying gss_ctx_id_t object be
              freed when an instance is destroyed.
              """
        
        self.__secureModule = SecuritySingleton()
        
        if handle == None:
            self._handle = gsic.create_context()
        else:
            self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
              Destroys an instance.
              
              Deletes the underlying gss_ctx_id_t object, and free's the
              memory if the instance owns the underlying handle.
              """
        
        if self._free:
            major, minor = gsic.delete_context(self._handle)
            if not major == gsic.GSS_S_COMPLETE:
		list = gsic.display_status(major, minor)
		ex = apply(GSSContextException, list)
		raise ex
                
        return

    def inquire(self):
        """
	Obtain information about the context.
              
	**Returns:**
	A 7-tuple containing:
	A B{GSSName} object containing the initiators name.
	A B{GSSName} object with the acceptors name.              
	An B{int} containing the number of seconds the context
	will remain valid for.                
	A B{string} containing a SWIG'ized pointer to a static
	gss_OID struct containing the mechanism type.
	An B{int} containing the context flags.
	A B{bool} indicating if the connection was initiated
	locally.
	A B{bool} indicating if the connection is currently open.

	**Raises:**

	A GSSContextException is raised if an error occurs in the
	underlying gss-api call.
	"""
        
        tuple = gsic.inquire_context(self._handle)
        if not tuple[0] == gsic.GSS_S_COMPLETE:
	    list = gsic.display_status(tuple[0], tuple[1])
	    ex = apply(GSSContextException, list)
	    raise ex
        return GSSName(tuple[2]), GSSName(tuple[3]), tuple[4], \
	       tuple[5], tuple[6], tuple[7], tuple[8]

    def init_context(self, init_cred, target_name, input_token):
        """
              Tries to establish a secuirty context with a server
              
              **Arguments**
        
              - *init_cred* is a GSSCred object which contains the client's credential handle
        
              - *target_name* is a string
        
              - *input_token* is a GSSBuffer object

              **Returns:**
              major and minor status error codes from the GSSAPI.
              A string of tokens which represent credentials from the client who is initiating
              the context.
              
              **Raises:**

              A GSSContextException is raised if an error occurs in the
              underlying gss-api call.
              """
        
        major, minor, token_string, handle = gsic.init_sec_context(self._handle,
                                                                   init_cred.get_handle(),
                                                                   target_name.get_handle(),
                                                                   input_token.get_handle())
        
        if major != gsic.GSS_S_COMPLETE and major != gsic.GSS_S_CONTINUE_NEEDED:
            list = gsic.display_status(major, minor)
	    ex = apply(GSSContextException, list)
	    raise ex

        if major == gsic.GSS_S_COMPLETE:
            self._handle = handle
        
        return major, minor, token_string
    
    def accept_context(self,acceptor_cred, token_string):
        """
              **Arguments**
        
              - *acceptor_cred* is a GSSCred object which contains the credential that the acceptor
              will use to authenticate itself
        
              - *token_string* is a token received from the corresponding init_sec call

              **Returns**
              major and minor status error codes from the GSSAPI.
              A string of tokens which represent credentials from the client who is initiating
              the context.
              
              **Raises:**

              A GSSContextException is raised if an error occurs in the
              underlying gss-api call.
              """
      
        major, minor,token_string, handle = gsic.accept_sec_context(self._handle,
                                                                    acceptor_cred.get_handle(),
                                                                    token_string.get_handle())
        
        if major != gsic.GSS_S_COMPLETE and major != gsic.GSS_S_CONTINUE_NEEDED:
            list = gsic.display_status(major, minor)
	    ex = apply(GSSContextException, list)
	    raise ex

        if major == gsic.GSS_S_COMPLETE:
            self._handle = handle
        
        return major, minor, token_string
       
    
    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################
    def get_handle(self):
        return self._handle
    

class GSSCred:
    """
       A class for the underlying gss_cred_id_t object.
       """

    def __init__(self, handle=None, free=1):
        """
              Constructs an instance.

              Each instance wraps a gss_cred_id_t object.

              **Arguments:**

              - *handle* A SWIG'ized pointer to a gss_cred_id_t object.

              - *free* Should the underlying gss_cred_id_t object be
              freed when an instance is destroyed.
              """
        
        self.__secureModule = SecuritySingleton()
        if handle==None:
            handle = gsic.get_no_cred_const_ptr()
        
        self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
              Destroys an instance.
              
              Deletes the underlying gss_cred_id_t object, and free's the
              memory if the instance owns the underlying handle.
              """
        
        if self._free:
            major, minor = gsic.release_cred(self._handle)
            if not major == gsic.GSS_S_COMPLETE:
		list = gsic.display_status(major, minor)
		ex = apply(GSSCredException, list)
		raise ex
        else:
            gsic.free_cred_ptr(self._handle)
            
        
        return

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################
    def get_handle(self):
        return self._handle   


    def acquire_cred(self, name=None, desired_mechs=None, usage=None):
        """
              Acquires a credential, for example the one created by grid_proxy_init in /tmp/x509uid
        
              **Arguments**
                     
              - *name*
              a GSSName object
              
              - *desired_mechs* 
              a GSSMechs object 
              
              - *usage*
              a GSSUsage object

              *Returns*
              None

              **Raises**
              A GSSCredException is thrown if there is a problem acquiring the credential
              """
        if name == None:
            name = GSSName()

        if desired_mechs == None:
            desired_mechs = GSSMechs()
            
        if usage == None:
            usage = GSSUsage()
            
        major, minor, cred = gsic.acquire_cred(name.get_handle(),
                                               desired_mechs.get_handle(),
                                               usage.get_handle())
        
        if not major == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(major, minor)
            ex = apply(GSSCredException, list)
            raise ex
        
        self._handle = cred
        
    def export_external(self):
        """
              Saves the credential so it can be checkpointed and
              imported by import_external
              
              **Returns**
              A string pointing the location of the expored credential

              **Raises**
              A GSSCredException is raised if there is a problem in the underlying cred
              """

        tuple = gsic.export_cred(self._handle, gsic.GSS_EXPORT_CRED_EXTERNAL)
        if not tuple[0] == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(tuple[0], tuple[1])
	    ex = apply(GSSContextException, list)
	    raise ex
        return tuple[2]

    def import_external(self, credHandle):
        """
              Imports an external credential

              **Arguments**

              - *credHandle* - A string describing the type and location of the credential to import

              **Returns**
              A GSSCred Object

              **Raises**

              A GSSCredException is raised if the the credential cannot be imported
              """
        
        major, minor, cred, time = gsic.import_cred(gsic.GSS_EXPORT_CRED_EXTERNAL,
                                                    credHandle, 0)
        if not major == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(major, minor)
            ex = apply(GSSCredException, list)
            raise ex
        return GSSCred(cred)
    
    def inquire_cred(self):
        """
              Retrieves information about the credential
              
              **Raises**
              a is thrown if there is a problem inquiring
              about the credential
              
              **Returns**
              A tuple of the lifetime in seconds and GSSName
              object

              **Raises**

              A GSSCredException is raised if the the credential cannot be imported
              """
        
        major, minor, life, name = gsic.inquire_cred(self._handle)
        if not major == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(major, minor)
            ex = apply(GSSCredException, list)
            raise ex
        return (life, GSSName(name))


        
        
class X509_Request:
    """
       A simple class for the underlying X509_REQ* objects.
       
       """

    def __init__(self, handle=None, free=1):
        """
              Constructs an instance.

              Each instance wraps X509_REQ* objects.

              **Arguments:**
              
              - *handle* A SWIG'ized pointer to a X509_REQ* object.
              
              - *free* Should the underlying objects be
              freed when an instance is destroyed.
              """
        
        if handle == None:
            handle = gsic.new_X509()
        self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
              Destroys an instance.
              
              Deletes the underlying gss_cred_id_t object, and free's the
              memory if the instance owns the underlying handle.
              """
        
        if self._free:
            gsic.free_X509(self._handle)
        return
    
class EVP_Key:
    """
        A class for the underlying  EVP_PKEY* objects.
        
        """

    def __init__(self, handle=None, free=1):
        """
              Constructs an instance.
              
              Each instance wraps X509_REQ* EVP_PKEY* objects.

              **Arguments:**

              - *handle* A SWIG'ized pointer to a EVP_PKEY* object.
            
              - *free* Should the underlying objects be
              freed when an instance is destroyed.
              """
        
        if handle == None:
            handle = gsic.new_EVP_PKEY()

        self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
              Destroys an instance.

              Deletes the underlying gss_cred_id_t object, and free's the
              memory if the instance owns the underlying handle.
              """
        
        if self._free:
            gsic.free_EVP_PKEY(self._handle)
        return

             
class GSSName:
    """
       A class for the underlying gss_name_t object.
       """

    def __init__(self, handle=None, free=1):
        """
              Constructs an instance.
              
              Each instance wraps a gss_name_t object.

              **Arguments:**

              - *handle* An optional argument that, if present, should
              contain a SWIG'ized pointer to a gss_name_t object.
              It defaults to a pointer to a  GSS_C_NO_NAME
              
              - *free* Should the underlying gss_name_t object be
              freed when an instance is destroyed.

              **Returns**
              None

              **Raises**
              None
              """
        
        if handle == None:
            handle = gsic.get_default_name()
        self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
              Destroys an instance.
              
              Deletes the underlying gss_name_t object, and free's the
              memory if the instance owns the underlying handle.

              **Raises:**

              A GSSNameException is raised if an error occurs
              in the underlying gss-api call.
              """
        
        if not self._handle == None:
            if self._free:
                major, minor = gsic.release_name(self._handle)
            if not major == gsic.GSS_S_COMPLETE:
                list = gsic.display_status(major, minor)
		ex = apply(GSSContextException, list)
		raise ex
        return

    def display(self):
        """
              Returns the name as a string.

              **Raises:**

              A GSSNameException is raised if an error occurs
              in the underlying gss-api call.

              **Returns:**
              
              A string containing the name.
              """
        
        major, minor, name = gsic.display_name(self._handle)
        if not major == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(major, minor)
	    ex = apply(GSSContextException, list)
	    raise ex
        return name

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################
    def get_handle(self):
        return self._handle


class GSSMechs:
    """
       A  class for the underlying gss_OID_set object.

       
       """

    def __init__(self, handle=None, free=1):
        """
              **Arguments**

              - *handle* A SWIG'ized pointer to a gss_OID_set object
              The default case creates a new gss_OID_set object for you

              - *free* If set to zero then the caller is reponsible for freeing the
              hande object.
              The default case frees the memory of the handle when the
              __del__ method is called

              **Raises**
              GSSMechsException if there is a problem initializing the
              underlying structure
               """
        
        if handle == None:
            major, minor, handle = gsic.create_empty_oid_set()
            if not major == gsic.GSS_S_COMPLETE:
                list = gsic.display_status(major, minor)
                ex = apply(GSSMechsException, list)
		raise ex
        self._handle = handle
        self._free = free

    def __del__(self):
        """
              Deletes the object and frees the memory of the handle variable if
              free is set
              
              **Raises**
              GSSMechsException if there is a problem freeing the
              underlying structure
              """
        
        if self._free:
            major, minor = gsic.release_oid_set(self._handle)
            if not major == gsic.GSS_S_COMPLETE:
                list = gsic.display_status(major, minor)
                ex = apply(GSSMechsException, list)
                raise ex 
            
    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################        
    def get_handle(self):
        return self._handle

class GSSUsage:
    """
       A  class for the underlying gss_cred_usage_t object.
       """
    
    def __init__(self, handle=None, free=1):
        """
              Defaults to GSS_C_BOTH
              """
        
        if handle == None:
            handle = gsic.get_usage_both()
        self._handle = handle
        self._free = free

    def __del__(self):
        if self._free:
            gsic.free_usage(self._handle)

    def set_usage_initiate(self):
        """
              Sets the underlying handle to a GSS_C_INITIATE handle
              """
        
        if self._free:
            gsic.free_usage(self._handle)

        self._handle = gsic.get_usage_initiate()

    def set_usage_accept(self):
        """
              Sets the underlying handle to a GSS_C_ACCEPT handle
              """
        
        if self._free:
            gsic.free_usage(self._handle)

        self._handle = gsic.get_usage_accept()

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################  
    def get_handle(self):
        return self._handle
        
class GSSBuffer:
    """
       A class for the underlying gss_buffer_t object
       """
            

    def __init__(self, handle=None, free=1):
        """
              defaults to GSS_C_NO_BUFFER
              """

        if handle == None:
            handle = gsic.get_empty_buffer()
        self._handle = handle
        self._free = free


    def __del__(self):
        if self._free:
            major, minor = gsic.free_buffer(self._handle)
                

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################  
    def get_handle(self):
        return self._handle

    def set_buffer(self, contents):
        """
              **Args**
              
              - *contents* a string which you wish to set the underlying handle to
        
              **Returns**
              None
        
              **Raises**
              None
              """
        
        gsic.free_buffer(self._handle)
        self._handle = gsic.set_buffer(contents)


    def get_string(self):
        """
              Returns the contents of the buffer as a python string
              """
        
        return gsic.get_buffer_string(self._handle)
              
