############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
Version 2.2
"""


from distutils.core import setup, Extension
from distutils.spawn import spawn
from distutils.sysconfig import get_python_inc
from distutils.sysconfig import get_config_vars
from distutils.util import get_platform
from os import environ, popen4
from os.path import isfile
import sys, string, re, exceptions, os, glob

GLOBUS_LOCATION = None
FLAVOR          = "gcc32dbgpthr"
RUN_SWIG        = 0
DEBUG           = 0
RUN_HAPPYDOC    = 0
RUN_EPYDOC      = 0
RUN_EPYDOCIFY   = 1
GLOBUS_LIBS     = None
GLOBUS_PACKAGES = None
SWIG_ARGS = ["-python", "-noproxy", "-DHAVE_CONFIG_H",
             "-I%s" % get_python_inc(plat_specific=1)]
GLOBUS_DEPS = {"util" : "globus_common"}
MAKEFILE_HEADER = None
BUILT_LIST = []
FAILED_LIST = []
MODULE_LIST = ['util','ftpClient','ftpControl','gassCache', 'gassCopy','gassServerEZ', 
               'gassTransfer','gramClient','io','myProxy','replicaCatalog', 'replicaManagement',
		'rlsClient','security']

# Get GLOBUS_LOCATION, exit if it doesn't exist
try:
    GLOBUS_LOCATION = environ["GLOBUS_LOCATION"]
except KeyError:
    print "GLOBUS_LOCATION must be set before installing pyGlobus"
    sys.exit(-1)

# Globus-2.2 requires GPT_LOCATION be set before running
# globus-makefile-header
try:
    GPT_LOCATION = environ["GPT_LOCATION"]
except KeyError:
    print "GPT_LOCATION must be set before installing pyGlobus"
    sys.exit(-1)
    
# parse special command line options and remove them
# this way distutils won't attempt to parse them.
args = sys.argv[1:]
for arg in args:
    if string.find(arg, '--flavor=') == 0:
        FLAVOR = string.split(arg, '=')[1]
        sys.argv.remove(arg)
    elif string.find(arg, '--run-swig') == 0:
        RUN_SWIG = 1
        sys.argv.remove(arg)
    elif string.find(arg, '--run-happydoc') == 0:
        RUN_HAPPYDOC = 1
        sys.argv.remove(arg) 
    elif string.find(arg, '--run-epydoc') == 0:
        RUN_EPYDOC = 1
        sys.argv.remove(arg)
    elif string.find(arg, '--debug') == 0:
	DEBUG = 1
	sys.argv.remove(arg)
    elif string.find(arg, '--with-modules') == 0:
        modules = string.split(arg, '=')[1]
        mList = []
        while not modules == '':
            indx = modules.find(',')
            if indx == -1:
                mName = modules
                modules = ''
            else:
                mName = modules[:indx]
                modules = modules[indx +1:]
            mList.append(mName)
        MODULE_LIST = mList
        sys.argv.remove(arg)
        


#Find globus-makefile-header. If unable to find it, exit
if isfile("%s/sbin/globus-makefile-header" % GLOBUS_LOCATION):
    MAKEFILE_HEADER = "%s/sbin/globus-makefile-header -flavor=%s" % (GLOBUS_LOCATION, FLAVOR)
elif isfile("%s/bin/globus-makefile-header" % GLOBUS_LOCATION):
    MAKEFILE_HEADER = "%s/bin/globus-makefile-header -flavor=%s" % (GLOBUS_LOCATION, FLAVOR)
else:
    print "Unable to locate globus-makefile-header in either $GLOBUS_LOCATION/[bin,sbin]. This must be found before installing pyGlobus."
    sys.exit(-2)
    
HAPPYDOC = ["happydoc", "-d", "./doc", "pyGlobus", "formatter_levelOneHeadingBackgroundColor='#CCCCCC'", "formatter_levelOneHeadingForegroundColor='000000'", "formatter_levelTwoHeadingBackgroundColor='#CCCCCC'", "formatter_levelTwoHeadingForegroundColor='000000'", "./pyGlobus"]
errRegEx = r".*ERROR.*|\s*No packages were found.*"
incRegEx = r"\s*GLOBUS_INCLUDES\s*=\s*"
cflagsRegEx = r"\s*GLOBUS_CFLAGS\s*=\s*"
pkgLibRegEx = r"\s*GLOBUS_PKG_LIBS\s*=\s*"
ldFlagsRegEx = r"\s*GLOBUS_LDFLAGS\s*=\s*"
libsRegEx = r"\s*GLOBUS_LIBS\s*=\s*"
ERR_PATTERN = re.compile(errRegEx)
INC_PATTERN = re.compile(incRegEx)
CFLAGS_PATTERN = re.compile(cflagsRegEx)
PKGLIBS_PATTERN = re.compile(pkgLibRegEx)
LDFLAGS_PATTERN = re.compile(ldFlagsRegEx)
LIBS_PATTERN = re.compile(libsRegEx)

class DependencyException(exceptions.Exception):
    pass

def build_module_lists(mList = MODULE_LIST):
    print "Checking for which modules to build"
    swigList = []
    pyModList = []
    eModList = []
    for module in mList:
        if module == 'util':
            check_util(pyModList, eModList, swigList)
        elif module == 'ftpClient':
            check_ftpClient(pyModList, eModList, swigList)
        elif module == 'ftpControl':
            check_ftpControl(pyModList, eModList, swigList)
        elif module == 'gassCache':
            check_gassCache(pyModList, eModList, swigList)
        elif module == 'gassCopy':
            check_gassCopy(pyModList, eModList, swigList)
        elif module == 'gassServerEZ': 
            check_gassServerEZ(pyModList, eModList, swigList)
        elif module == 'gassTransfer':
            check_gassTransfer(pyModList, eModList, swigList)
        elif module == 'gramClient':
            check_gramClient(pyModList, eModList, swigList)
        elif module == 'io':
            check_io(pyModList, eModList, swigList)
        elif module == 'myProxy':
            check_myProxy(pyModList, eModList, swigList)
        elif module == 'replicaCatalog':
            check_replicaCatalog(pyModList, eModList, swigList)
        elif module == 'replicaManagement':
            check_replicaManagement(pyModList, eModList, swigList)
        elif module == 'rlsClient':
            check_rlsClient(pyModList, eModList, swigList)
        elif module == 'security':
            check_security(pyModList, eModList, swigList)
        else:
            raise DependencyException , "bad name: %s in module_list" %(module)
    return pyModList, eModList, swigList

def check_util(pyModList, eModList, swigList):
    print "Checking the util module"
    dep = "globus_common"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the util package."
        print "This package is required to build all of pyGlobus."
        print "The following is the error message from globus-makefile-header."
        print ex.args
        print "Now exiting from the build."
        sys.exit(-1)

    # noah TODO: add try/except here
    
    e = Extension("utilc", ["src/util_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("util.i", "util_wrap.c"))
    pyModList.append("pyGlobus.util")
    pyModList.append("pyGlobus.logUtil")
    pyModList.append("pyGlobus.test.test_util")
    pyModList.append("pyGlobus.test.test_pyGlobus")
    pyModList.append("pyGlobus.test.config") 
    BUILT_LIST.append("util")
    print "The util module will be built"
    return

def check_rlsClient(pyModList, eModList, swigList):
    print "Checking the rlsClient module"
    dep = "globus_rls_client"
    try:
        libList, incList, cFlagsList, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the rlsClient package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("rlsClient")
        return
    e = Extension("rlsClientc", ["src/rlsClient_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=cFlagsList,
                  libraries=libList)
    eModList.append(e)
    swigList.append(("rlsClient.i", "rlsClient_wrap.c"))
    pyModList.append("pyGlobus.rlsClient")
    pyModList.append("pyGlobus.test.test_RlsClient")
    pyModList.append("pyGlobus.test.test_rlsClient")
    BUILT_LIST.append("rlsClient")
    print "The rlsClient module will be built"
    return

def check_myProxy(pyModList, eModList, swigList):
    print "Checking the myProxy module"
    dep = "myproxy"
    try:
        libList, incList, cFlagsList, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the rlsClient package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("myProxy")
        return
    e = Extension("myProxyc", ["src/myProxy_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=cFlagsList,
                  libraries=libList)
    eModList.append(e)
    swigList.append(("myProxy.i", "myProxy_wrap.c"))
    pyModList.append("pyGlobus.myProxy")
    pyModList.append("pyGlobus.test.testMyProxy")
    BUILT_LIST.append("myProxy")
    print "The myProxy module will be built"
    return

def check_ftpClient(pyModList, eModList, swigList):
    print "Checking the ftpClient module"
    dep = "globus_ftp_client"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the ftpClient package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("ftpClient")
        return
    e = Extension("ftpClientc", ["src/ftpClient_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("ftpClient.i", "ftpClient_wrap.c"))
    pyModList.append("pyGlobus.ftpClient") 
    pyModList.append("pyGlobus.ftpClientAttr")
    pyModList.append("pyGlobus.ftpClientPlugin")
    pyModList.append("pyGlobus.test.test_FtpClient")
    pyModList.append("pyGlobus.test.test_FtpClientAttr")
    pyModList.append("pyGlobus.test.test_ftpClient")
    pyModList.append("pyGlobus.test.test_EasyFtpClient")
    pyModList.append("pyGlobus.test.test_FtpClientPlugin") 
    BUILT_LIST.append("ftpClient")
    print "The ftpClient module will be built"
    return

def check_ftpControl(pyModList, eModList, swigList):
    print "Checking the ftpControl module"
    dep = "globus_ftp_control"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the ftpControl package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("ftpControl")
        return
    e = Extension("ftpControlc", ["src/ftpControl_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("ftpControl.i", "ftpControl_wrap.c"))
    pyModList.append("pyGlobus.ftpControl")
    pyModList.append("pyGlobus.test.test_FtpControl")
    pyModList.append("pyGlobus.test.test_ftpControl")
    BUILT_LIST.append("ftpControl")
    print "The ftpControl module will be built"
    return

def check_gassCache(pyModList, eModList, swigList):
    print "Checking the gassCache module"
    dep = "globus_gass_cache"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep) 
    except DependencyException, ex:
        print "Missing dependency to build the gassCache package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassCache")
        return
    major, minor = check_version(dep)
    e = Extension("gassCachec", ["src/gassCache_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    if major == 4:
	print "using version %i.%i" % (major, minor)
	swigList.append(("gassCache-4.0.i", "gassCache_wrap.c"))
    elif major == 5:
        print "using version %i.%i" % (major, minor)
	swigList.append(("gassCache-5.0.i", "gassCache_wrap.c"))
    else:
	print "WARNING globus_gass_cache version %i.%i" % (major, minor), " installed. Using version 5.0"
	swigList.append(("gassCache-5.0.i", "gassCache_wrap.c"))
    pyModList.append("pyGlobus.gassCache")
    pyModList.append("pyGlobus.test.test_GassCache")
    pyModList.append("pyGlobus.test.test_gassCache")
    BUILT_LIST.append("gassCache")
    print "The gassCache module will be built"
    return

def check_gassCopy(pyModList, eModList, swigList):
    print "Checking the gassCopy module"
    dep = "globus_gass_copy"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gassCopy package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassCopy")
        return
    e = Extension("gassCopyc", ["src/gassCopy_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassCopy.i", "gassCopy_wrap.c"))
    pyModList.append("pyGlobus.gassCopy")
    pyModList.append("pyGlobus.test.test_GassCopy")
    pyModList.append("pyGlobus.test.test_gassCopy")
    BUILT_LIST.append("gassCopy")
    print "The gassCopy module will be built"
    return


def check_gassServerEZ(pyModList, eModList, swigList):
    print "Checking the gassServerEZ module"
    dep = "globus_gass_server_ez"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gassServerEZ package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassServerEZ")
        return
    e = Extension("gassServerEZc", ["src/gassServerEZ_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassServerEZ.i", "gassServerEZ_wrap.c"))
    pyModList.append("pyGlobus.gassServerEZ")
    pyModList.append("pyGlobus.test.test_GassServerEZ")
    pyModList.append("pyGlobus.test.test_gassServerEZ")
    BUILT_LIST.append("gassServerEZ")
    print "The gassServerEZ module will be built"
    return

def check_gassTransfer(pyModList, eModList, swigList):
    print "Checking the gassTransfer module"
    dep = "globus_gass_transfer"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gassTransfer package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassTransfer")
        return
    e = Extension("gassTransferc", ["src/gassTransfer_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassTransfer.i", "gassTransfer_wrap.c"))
    pyModList.append("pyGlobus.gassTransfer")
    pyModList.append("pyGlobus.test.test_GassTransfer")
    pyModList.append("pyGlobus.test.test_gassTransfer")
    BUILT_LIST.append("gassTransfer")
    print "The gassTransfer module will be built"
    return

def check_gramClient(pyModList, eModList, swigList):
    print "Checking the gramClient module"
    dep = "globus_gram_client"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gramClient package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gramClient")
        return
    major, minor = check_version(dep)
    print "major: ", major 
    e = Extension("gramClientc", ["src/gramClient_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    if major == 3:
        print "using version %i.%i" % (major, minor)
        swigList.append(("gramClient-3.0.i", "gramClient_wrap.c"))
    elif major == 4:
        print "using version %i.%i" % (major, minor)
        swigList.append(("gramClient-4.0.i", "gramClient_wrap.c"))
    else:
        print "WARNING globus_gram_client version %i.%i" % (major, minor), " installed. Using version 4.0"
        swigList.append(("gramClient-4.0.i", "gassClient_wrap.c"))
    pyModList.append("pyGlobus.gramClient")
    pyModList.append("pyGlobus.rsl")
    pyModList.append("pyGlobus.test.test_GramClient")
    pyModList.append("pyGlobus.test.test_gramClient")
    pyModList.append("pyGlobus.test.testGramClient")
    BUILT_LIST.append("gramClient")
    print "The gramClient module will be built"
    return

def check_io(pyModList, eModList, swigList):
    print "Checking the io module"
    dep = "globus_io"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the io package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("io")
        return
    e = Extension("ioc", ["src/io_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("io.i", "io_wrap.c"))
    pyModList.append("pyGlobus.io")
    pyModList.append("pyGlobus.asynGSI")
    pyModList.append("pyGlobus.GSISOAP")
    pyModList.append("pyGlobus.GSIasyncore") 
    pyModList.append("pyGlobus.test.test_GSITcpClient")
    pyModList.append("pyGlobus.test.test_io")
    pyModList.append("pyGlobus.test.test_GSITCPSocket")
    pyModList.append("pyGlobus.test.test_GSITCPSocketServer")
    pyModList.append("pyGlobus.test.test_TcpClient")
    pyModList.append("pyGlobus.test.test_TcpServer") 
    BUILT_LIST.append("io")
    print "The io module will be built"
    return

def check_replicaCatalog(pyModList, eModList, swigList):
    print "Checking the replicaCatalog module"
    dep = "globus_replica_catalog"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the replicaCatalog package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("replicaCatalog")
        return
    e = Extension("replicaCatalogc", ["src/replicaCatalog_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("replicaCatalog.i", "replicaCatalog_wrap.c"))
    pyModList.append("pyGlobus.replicaCatalog")
    pyModList.append("pyGlobus.test.test_Collection")
    pyModList.append("pyGlobus.test.test_replicaCatalog")
    BUILT_LIST.append("replicaCatalog")
    print "The replicaCatalog module will be built"
    return

def check_replicaManagement(pyModList, eModList, swigList):
    print "Checking the replicaManagement module"
    dep = "globus_replica_management"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the replicaManagement package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("replicaManagement")
        return
    e = Extension("replicaManagementc", ["src/replicaManagement_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("replicaManagement.i", "replicaManagement_wrap.c"))
    pyModList.append("pyGlobus.replicaManagement")
    pyModList.append("pyGlobus.test.test_Session")
    pyModList.append("pyGlobus.test.test_replicaManagement")
    BUILT_LIST.append("replicaManagement")
    print "The replicaManagement module will be built"
    return

def check_security(pyModList, eModList, swigList):
    print "Checking the security module"
    dep = "globus_io"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the security package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("security")
        return
    
        
    e = Extension("gsic", ["src/gsi_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e) 
    swigList.append(("gsi.i", "gsi_wrap.c"))
    pyModList.append("pyGlobus.security")
    pyModList.append("pyGlobus.test.test_GSSCred")
    pyModList.append("pyGlobus.test.test_GSSContext")
    pyModList.append("pyGlobus.test.test_GSSSimpleClasses")
    pyModList.append("pyGlobus.test.test_security")
    pyModList.append("pyGlobus.test.test_GridProxyInit")
    BUILT_LIST.append("security")
    print "The security module will be built"
    return

def get_makefile_output(dep):
    pLibsStr = None
    eLibsStr = None
    cFlagsStr = None
    includeStr = None
    ldFlagsStr = None
    libList = []
    incList = []
    ldFlagsList = []
    inStrm, outStrm = popen4("%s %s" % (MAKEFILE_HEADER, dep))
    for line in outStrm.readlines():
        m = ERR_PATTERN.match(line)
        if m:
            raise DependencyException(line)
        m = INC_PATTERN.match(line)
        if m:
            includeStr = INC_PATTERN.split(line)[1]
            for str in string.split(includeStr):
                s = string.strip(str)[2:]
                incList.append(s)
            continue
        m = CFLAGS_PATTERN.match(line)
        if m:
            cFlagsStr = CFLAGS_PATTERN.split(line)[1]
            continue
        m = PKGLIBS_PATTERN.match(line)
        if m:
            pLibsStr = PKGLIBS_PATTERN.split(line)[1]
            i = 0
            for str in string.split(pLibsStr):
                s = string.strip(str)[2:]
                libList.insert(i, s)
                i += 1
            continue
        m = LDFLAGS_PATTERN.match(line)
        if m:
            ldFlagsStr = LDFLAGS_PATTERN.split(line)[1]
            for str in string.split(ldFlagsStr):
                s = string.strip(str)[2:]
                ldFlagsList.append(s)
            continue
        m = LIBS_PATTERN.match(line)
        if m:
            eLibsStr = LIBS_PATTERN.split(line)[1]
            for str in string.split(eLibsStr):
                s = string.strip(str)[2:]
                libList.append(s)
            continue
   
    if DEBUG:
	libList.append('wad')
    return libList, incList, cFlagsStr, ldFlagsList

def check_version(dep):
    GPT_QUERY = GPT_LOCATION+ "/sbin/gpt-query"
    inStr, outStr = popen4("%s %s"% (GPT_QUERY, dep))
    for line in outStr:
	if line.find(dep) != -1:
	    tmpstr = string.split(line, "ver: ")[1]
	    version = string.split(tmpstr, " cmp")[0]
            major, minor = string.split(version, '.')
	    return (int(major), int(minor))
	    
def make_data_list():
    dirList = os.listdir(os.getcwd())
    try:
	dirList.index("doc")
    except ValueError, ex:
	return []
    
    l = [('doc/pyGlobus', glob.glob('doc' + os.sep + '*')), 
        ('doc/pyGlobus' + os.sep + 'public', 
        glob.glob('doc' + os.sep + 'public' + os.sep + '*')), 
        ('doc/pyGlobus' + os.sep + 'private', 
        glob.glob('doc' + os.sep + 'private' + os.sep + '*'))]
    try:
        l[0][1].remove('doc/public')
        l[0][1].remove('doc/private')
    except:
        print 'No doc/public and/or doc/private to remove!'
    return l


# Distutils doesn't currently work with recent 1.3.x
# versions of swig.
def run_swig(swigList):
    for s in swigList:
        cmd = ["swig"] + SWIG_ARGS + ["-o", "./src/%s" % (s[1]),
                                      "./src/%s" % (s[0])]
        spawn(cmd, verbose=1)


pyModList, eModList, swigList = build_module_lists()
dataList = make_data_list()

if RUN_SWIG:
    run_swig(swigList)


def run_epydocify():  # will copy epydocified files to $PYTHONPATH/pyGlobus
                      # but keep pyGlobus/ source the same!
    try:    spawn(['mkdir', 'pyGlobus/tmp'])    # i hope people didnt make a tmp directory in pyGlobus
    except: print 'whoopsie!'
    flist = glob.glob('pyGlobus/*.py')             #find all files in pyGlobus directory and copy to tmp
    for x in flist:
        try: spawn(['cp', x, "pyGlobus/tmp"])
        except: print 'convert to pass!'
    runstr = [sys.executable, 'scripts/epydocify.py']   # run epydocify on all source in tmp dir
    runstr.extend(glob.glob('pyGlobus/tmp/*.py'))
    spawn(runstr,verbose=1)
    runstr = [sys.executable, 'scripts/epydocify.py', '--replace-old-source'] # run epydocify repsource on tmpdir
    flist = glob.glob('pyGlobus/tmp/*.py')
    try:
        flist.remove(glob.glob('pyGlobus/gasscache.py'))
        print flist
        print 'nnn!'
    except:
        print 'AAAARGH!'
    runstr.extend(flist)
    spawn(runstr,verbose=1)

    flist = glob.glob('pyGlobus/tmp/*.py')
    for x in flist:
        spawn(['cp',x, os.environ['PYTHONPATH']+'/pyGlobus/'],verbose=1)

    try:
        for x in flist:
            spawn(['rm','-rf', x])
        spawn(['rmdir', 'pyGlobus/tmp'])
    except: pass
        

def run_epydoc():                # this gets run on the $PYTHONPATH/pyGlobus directory
    plat = get_platform()
    vers = sys.version_info

    # dir = 'build%slib.%s-%i.%i%spyGlobus%s*.py' % (os.sep, plat, vers[0],
    #                                                 vers[1], os.sep, os.sep)
    dir = os.environ['PYTHONPATH'] + '/pyGlobus/*.py'
    flist = glob.glob(dir)
    cmd = ['epydoc','--ignore-param-mismatch','-o', 'doc']
    
    for x in flist:
        x = x.replace(os.environ['PYTHONPATH'],'$PYTHONPATH')
    cmd.extend(flist)
    spawn(cmd, verbose=1)
    

def run_happydoc():
    plat = get_platform()
    vers = sys.version_info
    dir = 'build%slib.%s-%i.%i%spyGlobus%s*.py' % (os.sep, plat, vers[0],
                                                   vers[1], os.sep, os.sep)
    flist = glob.glob(dir)
    cmd = ['happydoc', '-d', 'doc', '-t', 'pyGlobus']
    cmd.extend(flist)
    spawn(cmd,verbose=1)
    return
                            
        

                                                    
#Remove flags from the compile line that through lots
# of warnings.
cv = get_config_vars()
opts = cv["OPT"]
opts = opts.replace("-Wall", "")
opts = opts.replace("-Wstrict-prototypes", "")
cv["OPT"] = opts

setup(name="pyGlobus", version="gt-2.2.x-1.0.2", description="Python Globus Toolkit wrapper",
      author="Keith Jackson", author_email="KRJackson@lbl.gov",
      url="http://www-itg.lbl.gov/gtg/pyGlobus/", 
      py_modules=pyModList, ext_package="pyGlobus", ext_modules=eModList,
      data_files=dataList)


if RUN_EPYDOCIFY and RUN_EPYDOC:
    run_epydocify()

if RUN_HAPPYDOC:
    run_happydoc()

if RUN_EPYDOC:
    run_epydoc()
                        
print "Built the following packages:\n"
for mod in BUILT_LIST:
    print mod
if not FAILED_LIST == []:
    print "\n\nFailed to build the following packages due to missing dependencies:\n"
    for mod in FAILED_LIST:
        print mod

